package in.slanglabs.assistants.retail;

import static in.slanglabs.assistants.retail.appStateAdapter.NavigationAppStateAdapter.getNavigationAppState;
import static in.slanglabs.assistants.retail.appStateAdapter.OrderManagementAppStateAdapter.getOrderManagementAppState;
import static in.slanglabs.assistants.retail.appStateAdapter.SearchAppStateAdapter.getSearchAppState;

import android.app.Activity;
import android.app.Application;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import androidx.annotation.NonNull;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import in.slanglabs.assistants.base.BaseUI;
import in.slanglabs.assistants.base.ContextualOnboardingListener;
import in.slanglabs.assistants.base.ContextualOnboardingResponse;
import in.slanglabs.assistants.base.ContextualOnboardingResponseReason;
import in.slanglabs.assistants.retail.appstates.CheckoutAppState;
import in.slanglabs.assistants.retail.appstates.FarewellAppState;
import in.slanglabs.assistants.retail.appstates.GreetingsAppState;
import in.slanglabs.assistants.retail.appstates.NavigationAppState;
import in.slanglabs.assistants.retail.appstates.OfferAppState;
import in.slanglabs.assistants.retail.appstates.OrderManagementAppState;
import in.slanglabs.assistants.retail.appstates.SearchAppState;
import in.slanglabs.assistants.retail.appstates.SmallTalkAppState;
import in.slanglabs.assistants.retail.appstates.UnsupportedAppState;
import in.slanglabs.assistants.retail.appstates.WaitingAppState;
import in.slanglabs.platform.SlangBuddy;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

/**
 * SlangRetailAssistantPlugin
 */
public class SlangRetailAssistantPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    private static final String TAG = "SlangRetailAssistant";

    private static final String CHANNEL_NAME = "plugins.flutter.io/slang_retail_assistant";
    private static final String CONFIG_ASSISTANT_ID = "assistantId";
    private static final String CONFIG_API_KEY = "apiKey";
    private static final String CONFIG_DEFAULT_LOCALE = "defaultLocale";
    private static final String CONFIG_REQUESTED_LOCALES = "requestedLocales";
    private static final String CONFIG_TRIGGER_POSITION = "triggerPosition";
    private static final String CONFIG_CUSTOM_TRIGGER_ENABLE = "enableCustomTrigger";
    private static final String CONFIG_DEFAULT_SUBDOMAIN = "defaultSubDomain";
    private static final String CONFIG_DEFAULT_USERJOURNEY = "defaultUserJourney";
    private static final String CONFIG_ENVIRONMENT = "environment";
    private static final String CONFIG_TRIGGER_STYLE = "triggerStyle";
    private static final String CONFIG_SURFACE_STYLE = "surfaceStyle";
    private static final String CONFIG_ONBOARDING_IDENTIFIER = "onboardingIdentifer";
    private static final String CONFIG_ONBOARDING_INFO_TITLE = "onboardingInfoTitle";
    private static final String CONFIG_ONBOARDING_INFO_DESCRIPTION = "onboardingInfoDescription";
    private static final String CONFIG_ONBOARDING_INFO_TITLE_SIZE = "onboardingInfoTitleSize";
    private static final String CONFIG_ONBOARDING_INFO_DESCRIPTION_SIZE = "onboardingInfoDescriptionSize";
    private static final String CONFIG_ONBOARDING_PRIMARY_TITLE = "onboardingPrimaryTitle";
    private static final String CONFIG_ONBOARDING_PRIMARY_DESCRIPTION = "onboardingPrimaryDescription";
    private static final String CONFIG_AUTOMATIC_APPEARANCE_LIMIT = "automaticAppearanceLimit";
    private static final String CONFIG_COACHMARK_VISIBILITY = "coachmarkVisibility";
    private static final String CONFIG_ENABLE_CUSTOM_TRIGGER = "enableCustomTrigger";
    private static final String CONFIG_DISABLE_ONBOARDING_LOCALE_SELECTION = "disableOnboardingLocaleSelection";
    private static final String CONFIG_DISABLE_ONBOARDING_INTRO = "disableOnboardingIntro";
    private static final String CONFIG_DISABLE_SURFACE_LOCALE_SELECTION = "disableSurfaceLocaleSelection";
    private static final String CONFIG_DISABLE_HELP_WINDOW = "disableHelpWindow";
    private static final String CONFIG_DISABLE_MUTE_BUTTON = "disableMuteButton";
    private static final String CONFIG_DISABLE_ITALIC_STYLE = "disableItalicStyle";
    private static final String CONFIG_DISABLE_COACHMARK = "disableCoachmark";
    private static final String CONFIG_INFO_TEXT_COLOR = "infoTextColor";
    private static final String CONFIG_INFO_ACCENT_COLOR = "infoAccentColor";
    private static final String CONFIG_BRAND_COLOR = "brandColor";
    //    private static final String CONFIG_CONFIG_OVERRIDES = "configOverrides";
//    private static final String CONFIG_APP_PROPERTIES = "appProperties";
    private static final String CONFIG_START_SPEECH_TIMEOUT_MILLIS = "startSpeechTimeoutMillis";
    private static final String CONFIG_ENABLE_SEARCH_LIST_FEEDBACK = "enableSearchListFeedback";
    private static final String CONFIG_ENABLE_NON_VOICE_COMPLETION_STATEMENTS = "enableNonVoiceCompletionStatements";
    private static final String CONFIG_DISABLE_ASR_PERMISSION_TALKBALK = "disableASRPermissionTalkback";
    private static final String CONFIG_MICROPHONE_PERMISSION_TEXT = "setMicrophonePermissionText";

    private MethodChannel channel;

    private Application application;
    private WeakReference<Activity> currentActivity;
    private SearchUserJourney searchUserJourney;
    private NavigationUserJourney navigationUserJourney;
    private OrderManagementUserJourney orderManagementUserJourney;

    /**
     * Default Constructor.
     *
     * <p>Use this constructor in an add to app scenario to gracefully handle activity and context
     * changes.
     */
    public SlangRetailAssistantPlugin() {
    }

    private SlangRetailAssistantPlugin(Application application) {
        this.application = application;
    }

    /**
     * Registers a plugin with the v1 embedding api {@code io.flutter.plugin.common}.
     *
     * <p>Calling this will register the plugin with the passed registrar. However plugins initialized
     * this way won't react to changes in activity or context, unlike {@link
     * SlangRetailAssistantPlugin}.
     */
    @SuppressWarnings("unused")
    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL_NAME);
        channel.setMethodCallHandler(new SlangRetailAssistantPlugin(registrar.activity().getApplication()));
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPlugin.FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), CHANNEL_NAME);
        if (flutterPluginBinding.getApplicationContext() instanceof Application) {
            application = (Application) flutterPluginBinding.getApplicationContext();
        }
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "initialize":
                final Map<String, Object> arguments = call.arguments();
                SlangRetailAssistantUtils.setConfigOptions(arguments);
                try {
                    String environment
                            = SlangRetailAssistantUtils.getStringConfig(CONFIG_ENVIRONMENT);
                    String defaultUserJourneyString
                            = SlangRetailAssistantUtils.getStringConfig(CONFIG_DEFAULT_USERJOURNEY);
                    String defaultSubdomainString
                            = SlangRetailAssistantUtils.getStringConfig(CONFIG_DEFAULT_SUBDOMAIN);

                    AssistantConfiguration.Builder configBuilder = new AssistantConfiguration.Builder()
                            .setAssistantId(SlangRetailAssistantUtils.getStringConfig(CONFIG_ASSISTANT_ID))
                            .setAPIKey(SlangRetailAssistantUtils.getStringConfig(CONFIG_API_KEY))
                            .setDefaultUserJourney(
                                    RetailUserJourney.fromString(
                                            defaultUserJourneyString == null ?
                                                    RetailUserJourney.SEARCH.toString()
                                                    : defaultUserJourneyString
                                    )
                            )
                            .setDefaultSubDomain(
                                    AssistantSubDomain.fromString(
                                            defaultSubdomainString == null ?
                                                    AssistantSubDomain.GROCERY.toString()
                                                    : defaultSubdomainString
                                    )
                            )
                            .setDefaultLocale(SlangRetailAssistantLocaleMap.getDefaultLocale(
                                    SlangRetailAssistantUtils.getStringConfig(
                                            CONFIG_DEFAULT_LOCALE)))
                            .setRequestedLocales(
                                    SlangRetailAssistantLocaleMap.getRequestedLocales(
                                            SlangRetailAssistantUtils.getStringListConfig(
                                                    CONFIG_REQUESTED_LOCALES)))
                            .enableCustomTrigger(SlangRetailAssistantUtils
                                    .getBooleanConfig(CONFIG_CUSTOM_TRIGGER_ENABLE))
                            .setUIPosition(SlangRetailAssistantUtils
                                    .getAssistantUIConfig(CONFIG_TRIGGER_POSITION))
                            .setTriggerStyle(SlangRetailAssistantUtils.getTriggerStyle(CONFIG_TRIGGER_STYLE))
                            .setSurfaceStyle(SlangRetailAssistantUtils.getSurfaceStyle(CONFIG_SURFACE_STYLE))
                            .setEnvironment(
                                    environment == null ?
                                            SlangRetailAssistant.Environment.STAGING
                                            : environment.equals("PRODUCTION") ?
                                            SlangRetailAssistant.Environment.PRODUCTION
                                            : SlangRetailAssistant.Environment.STAGING
                            );


                    if (SlangRetailAssistantUtils.getStringConfig(CONFIG_BRAND_COLOR) != null) {
                        configBuilder.setBrandColor(SlangRetailAssistantUtils.getStringConfig(CONFIG_BRAND_COLOR));
                    }
                    if (SlangRetailAssistantUtils.getStringConfig(CONFIG_ONBOARDING_IDENTIFIER) != null) {
                        configBuilder.setOnboardingIdentifer(SlangRetailAssistantUtils.getStringConfig(CONFIG_ONBOARDING_IDENTIFIER));
                    }
                    if (SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_INFO_TITLE) != null) {
                        Map<Locale, String> map = SlangRetailAssistantUtils.convertToLocaleMap(SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_INFO_TITLE));
                        configBuilder.setOnboardingInfoTitle(map);
                    }
                    if (SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_INFO_DESCRIPTION) != null) {
                        Map<Locale, String> map = SlangRetailAssistantUtils.convertToLocaleMap(SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_INFO_DESCRIPTION));
                        configBuilder.setOnboardingInfoDescription(map);
                    }
                    if (SlangRetailAssistantUtils.getDoubleConfig(CONFIG_ONBOARDING_INFO_TITLE_SIZE) != null) {
                        configBuilder.setOnboardingInfoTitleSize(SlangRetailAssistantUtils.getDoubleConfig(CONFIG_ONBOARDING_INFO_TITLE_SIZE).floatValue());
                    }
                    if (SlangRetailAssistantUtils.getDoubleConfig(CONFIG_ONBOARDING_INFO_DESCRIPTION_SIZE) != null) {
                        configBuilder.setOnboardingInfoDescriptionSize(SlangRetailAssistantUtils.getDoubleConfig(CONFIG_ONBOARDING_INFO_DESCRIPTION_SIZE).floatValue());
                    }
                    if (SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_PRIMARY_TITLE) != null) {
                        Map<Locale, String> map = SlangRetailAssistantUtils.convertToLocaleMap(SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_PRIMARY_TITLE));
                        configBuilder.setOnboardingPrimaryTitle(map);
                    }
                    if (SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_PRIMARY_DESCRIPTION) != null) {
                        Map<Locale, String> map = SlangRetailAssistantUtils.convertToLocaleMap(SlangRetailAssistantUtils.getMapConfig(CONFIG_ONBOARDING_PRIMARY_DESCRIPTION));
                        configBuilder.setOnboardingPrimaryDescription(map);
                    }
                    if (SlangRetailAssistantUtils.getIntegerConfig(CONFIG_AUTOMATIC_APPEARANCE_LIMIT) != null) {
                        configBuilder.setAutomaticAppearanceLimit(SlangRetailAssistantUtils.getIntegerConfig(CONFIG_AUTOMATIC_APPEARANCE_LIMIT));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_COACHMARK_VISIBILITY) != null) {
                        configBuilder.setCoachmarkVisibility(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_COACHMARK_VISIBILITY));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_ENABLE_CUSTOM_TRIGGER) != null) {
                        configBuilder.enableCustomTrigger(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_ENABLE_CUSTOM_TRIGGER));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ONBOARDING_LOCALE_SELECTION) != null) {
                        configBuilder.disableOnboardingLocaleSelection(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ONBOARDING_LOCALE_SELECTION));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ONBOARDING_INTRO) != null) {
                        configBuilder.disableOnboardingIntro(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ONBOARDING_INTRO));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_SURFACE_LOCALE_SELECTION) != null) {
                        configBuilder.disableSurfaceLocaleSelection(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_SURFACE_LOCALE_SELECTION));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_HELP_WINDOW) != null) {
                        configBuilder.disableHelpWindow(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_HELP_WINDOW));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_MUTE_BUTTON) != null) {
                        configBuilder.disableMuteButton(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_MUTE_BUTTON));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ITALIC_STYLE) != null) {
                        configBuilder.disableItalicStyle(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ITALIC_STYLE));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_COACHMARK) != null) {
                        configBuilder.disableCoachMark(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_COACHMARK));
                    }
                    if (SlangRetailAssistantUtils.getStringConfig(CONFIG_INFO_TEXT_COLOR) != null) {
                        configBuilder.setInfoTextColor(SlangRetailAssistantUtils.getStringConfig(CONFIG_INFO_TEXT_COLOR));
                    }
                    if (SlangRetailAssistantUtils.getStringConfig(CONFIG_INFO_ACCENT_COLOR) != null) {
                        configBuilder.setAccentColor(SlangRetailAssistantUtils.getStringConfig(CONFIG_INFO_ACCENT_COLOR));
                    }
//                    if (SlangRetailAssistantUtils.getMapConfig(CONFIG_CONFIG_OVERRIDES) != null) {
//                        Map<String, Object> configOverrides = SlangRetailAssistantUtils.convertToStringMap(SlangRetailAssistantUtils.getMapConfig(CONFIG_CONFIG_OVERRIDES));
//                        configBuilder.setConfigOverrides(configOverrides);
//                    }
//                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_APP_PROPERTIES) != null) {
//                        Map<String, Object> appProperties = SlangRetailAssistantUtils.convertToStringMap(SlangRetailAssistantUtils.getMapConfig(CONFIG_APP_PROPERTIES));
//                        configBuilder.setAppProperties(appProperties);
//                    }
                    if (SlangRetailAssistantUtils.getIntegerConfig(CONFIG_START_SPEECH_TIMEOUT_MILLIS) != null) {
                        configBuilder.setStartSpeechTimeoutMillis(SlangRetailAssistantUtils.getIntegerConfig(CONFIG_START_SPEECH_TIMEOUT_MILLIS));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_ENABLE_SEARCH_LIST_FEEDBACK) != null) {
                        if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_ENABLE_SEARCH_LIST_FEEDBACK))
                            configBuilder.enableSearchListFeedback();
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_ENABLE_NON_VOICE_COMPLETION_STATEMENTS) != null) {
                        configBuilder.enableNonVoiceCompletionStatements(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_ENABLE_NON_VOICE_COMPLETION_STATEMENTS));
                    }
                    if (SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ASR_PERMISSION_TALKBALK) != null) {
                        configBuilder.disableASRPermissionTalkback(SlangRetailAssistantUtils.getBooleanConfig(CONFIG_DISABLE_ASR_PERMISSION_TALKBALK));
                    }
                    if (SlangRetailAssistantUtils.getMapConfig(CONFIG_MICROPHONE_PERMISSION_TEXT) != null) {
                        Map<Locale, String> map = SlangRetailAssistantUtils.convertToLocaleMap(SlangRetailAssistantUtils.getMapConfig(CONFIG_MICROPHONE_PERMISSION_TEXT));
                        configBuilder.setMicrophonePermissionText(map);
                    }
                    if (application != null) {
                        SlangRetailAssistant.initialize(application, configBuilder.build());
                    } else if (currentActivity != null && currentActivity.get() != null) {
                        SlangRetailAssistant.initialize(currentActivity.get().getApplication(), configBuilder.build());
                    } else {
                        Map<String, String> params = new HashMap<>();
                        params.put("type", "Invalid call to initialize");
                        channel.invokeMethod("onAssistantInitFailure", params);
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setAction":
                try {

                    SlangRetailAssistant.setAppAction(new SlangRetailAssistant.AppAction() {
                        @Override
                        public SearchAppState onSearch(final SearchInfo searchInfo, SearchUserJourney searchUserJourney) {
                            SlangRetailAssistantPlugin.this.searchUserJourney = searchUserJourney;
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onSearch",
                                                SlangRetailAssistantUtils.mapSearchInfo(searchInfo));
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                            return new WaitingAppState();
                        }

                        @Override
                        public OrderManagementAppState onOrderManagement(final OrderInfo orderInfo, OrderManagementUserJourney orderManagementUserJourney) {
                            SlangRetailAssistantPlugin.this.orderManagementUserJourney = orderManagementUserJourney;
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onOrderManagement",
                                                SlangRetailAssistantUtils.mapOrderInfo(orderInfo));
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                            return new WaitingAppState();
                        }

                        @Override
                        public NavigationAppState onNavigation(final NavigationInfo navigationInfo, NavigationUserJourney navigationUserJourney) {
                            SlangRetailAssistantPlugin.this.navigationUserJourney = navigationUserJourney;
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onNavigation",
                                                SlangRetailAssistantUtils.mapNavigationInfo(navigationInfo));
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                            return new WaitingAppState();
                        }

                        @Override
                        public OfferAppState onOfferManagement(final OfferInfo offerInfo, OfferManagementUserJourney offerManagementUserJourney) {
                            return new UnsupportedAppState();
                        }

                        @Override
                        public CheckoutAppState onCheckOut(final CheckoutInfo checkoutInfo, CheckoutUserJourney checkoutUserJourney) {
                            return new UnsupportedAppState();
                        }

                        @Override
                        public SmallTalkAppState onSmallTalk(SmallTalkUserJourney.SmallTalkType smallTalkType, SmallTalkUserJourney smallTalkUserJourney) {
                            if (smallTalkType == SmallTalkUserJourney.SmallTalkType.GREETING) {
                                return new GreetingsAppState(GreetingsAppState.CONTINUE);
                            }
                            else if (smallTalkType == SmallTalkUserJourney.SmallTalkType.FAREWELL) {
                                return new FarewellAppState(FarewellAppState.COMPLETE);
                            }
                            return new UnsupportedAppState();
                        }

                        @Override
                        public void onAssistantError(final AssistantError assistantError) {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    Log.d(TAG, String.valueOf(assistantError));
                                    Map<String, Object> params = new HashMap<>();
                                    params.put("description", assistantError.getDescription());
                                    channel.invokeMethod("onAssistantError", params);
                                }
                            });
                        }
                    });
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }

                break;

            case "setLifecycleObserver":
                try {
                    SlangRetailAssistant.setLifecycleObserver(
                            new SlangRetailAssistant.LifecycleObserver() {
                                @Override
                                public void onCoachmarkAction(SlangRetailAssistant.CoachmarkType coachmarkType, SlangRetailAssistant.CoachmarkAction coachmarkAction) {
                                    final Map<String, String> coachMarkMap = new HashMap<>();
                                    coachMarkMap.put("coachmarkType", coachmarkType.name());
                                    coachMarkMap.put("coachmarkAction", coachmarkAction.name());
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                channel.invokeMethod("onCoachmarkAction",
                                                        coachMarkMap);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onAssistantInitSuccess() {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                channel.invokeMethod("onAssistantInitSuccess", null);
                                            } catch (
                                                    Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onAssistantInitFailure(final String description) {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                Map<String, String> params = new HashMap<>();
                                                params.put("type", description);
                                                channel.invokeMethod("onAssistantInitFailure", params);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onAssistantInvoked() {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                channel.invokeMethod("onAssistantInvoked", null);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onAssistantClosed(final boolean isCancelled) {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                Map<String, Boolean> params = new HashMap<>();
                                                params.put("isCancelled", isCancelled);
                                                channel.invokeMethod("onAssistantClosed", params);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onAssistantLocaleChanged(final Locale changedLocale) {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                Map<String, String> params = new HashMap<>();
                                                params.put("country", changedLocale.getCountry().toUpperCase());
                                                params.put("language", changedLocale.getLanguage());
                                                channel.invokeMethod("onAssistantLocaleChanged", params);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public boolean onUnrecognisedUtterance(final String utterance) {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                Map<String, String> params = new HashMap<>();
                                                params.put("utterance", utterance);
                                                channel.invokeMethod("onUnrecognisedUtterance", params);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                    return false;
                                }

                                @Override
                                public void onUtteranceDetected(final String utterance) {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                Map<String, String> params = new HashMap<>();
                                                params.put("utterance", utterance);
                                                channel.invokeMethod("onUtteranceDetected", params);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onOnboardingSuccess() {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                channel.invokeMethod("onOnboardingSuccess", null);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onOnboardingFailure() {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                channel.invokeMethod("onOnboardingFailure", null);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onMicPermissionDenied() {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                channel.invokeMethod("onMicPermissionDenied", null);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }

                                @Override
                                public void onMicPermissionGranted() {
                                    new Handler(Looper.getMainLooper()).post(new Runnable() {
                                        @Override
                                        public void run() {
                                            try {
                                                channel.invokeMethod("onMicPermissionGranted", null);
                                            } catch (Exception exception) {
                                                sendAssistantError(exception);
                                            }
                                        }
                                    });
                                }
                            });
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }

                break;

            case "startConversation":
                String userJourney = call.argument("userJourney");
                if (currentActivity != null && currentActivity.get() != null) {
                    if (userJourney != null) {
                        try {
                            SlangRetailAssistant.startConversation(
                                    RetailUserJourney.fromString(userJourney),
                                    currentActivity.get());
                        } catch (Exception exception) {
                            sendAssistantError(exception);
                        }
                    }
                }
                break;
            case "startContextualOnboarding":
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.startContextualOnboarding(new ContextualOnboardingListener() {
                            @Override
                            public void onResponse(
                                    ContextualOnboardingResponse contextualOnboardingResponse,
                                    ContextualOnboardingResponseReason contextualOnboardingResponseReason) {

                                final Map<String, String> contextualOnboardingParams = new HashMap<>();
                                contextualOnboardingParams.put("contextualOnboardingResponse", contextualOnboardingResponse.name());
                                contextualOnboardingParams.put("contextualOnboardingResponseReason", contextualOnboardingResponseReason.name());
                                new Handler(Looper.getMainLooper()).post(new Runnable() {
                                    @Override
                                    public void run() {
                                        try {
                                            channel.invokeMethod("onContextualOnboardingResponse",
                                                    contextualOnboardingParams);
                                        } catch (Exception exception) {
                                            sendAssistantError(exception);
                                        }
                                    }
                                });

                            }
                        });
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }

                break;
//          UI Methods
            case "showTrigger":
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.getUI().showTrigger(currentActivity.get());
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "hideTrigger":
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.getUI().hideTrigger(currentActivity.get());
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;
            case "expandTrigger":
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.getUI().expandTrigger();
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;
            case "collapseTrigger":
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.getUI().collapseTrigger();
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;
            case "setMode":
                String mode = SlangRetailAssistantUtils.getStringConfig("mode");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (mode.equals("LIGHT"))
                            SlangRetailAssistant.getUI().setMode(BaseUI.BaseMode.LIGHT);
                        else if (mode.equals("DARK"))
                            SlangRetailAssistant.getUI().setMode(BaseUI.BaseMode.DARK);
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "disableLocaleSelection":
                Boolean disable = SlangRetailAssistantUtils.getBooleanConfig("disable");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (disable != null)
                            SlangRetailAssistant.getUI().disableLocaleSelection(disable);
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "nudgeUser":
                Map<Object, Object> titleStringMap = SlangRetailAssistantUtils.getMapConfig("title");
                Map<Object, Object> descriptionStringMap = SlangRetailAssistantUtils.getMapConfig("description");

                Map<Locale, String> title = SlangRetailAssistantUtils.convertToLocaleMap(titleStringMap);
                Map<Locale, String> descriptionMap = SlangRetailAssistantUtils.convertToLocaleMap(descriptionStringMap);

                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.getUI().nudgeUser(currentActivity.get(), title, descriptionMap);
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setForegroundColorGradient":
                List<String> foregroundColors = SlangRetailAssistantUtils.getStringListConfig("colors");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (foregroundColors != null) {
                            SlangRetailAssistant.getUI().setForegroundColorGradient((String[]) foregroundColors.toArray());
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setBackgroundColorGradient":
                List<String> backgroundColors = SlangRetailAssistantUtils.getStringListConfig("colors");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (backgroundColors != null) {
                            SlangRetailAssistant.getUI().setBackgroundColorGradient((String[]) backgroundColors.toArray());
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setWaveVelocity":
                Double velocity = SlangRetailAssistantUtils.getDoubleConfig("velocity");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (velocity != null) {
                            SlangRetailAssistant.getUI().setWaveVelocity(velocity.floatValue());
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setTriggerSize":
                Integer width = SlangRetailAssistantUtils.getIntegerConfig("width");
                Integer height = SlangRetailAssistantUtils.getIntegerConfig("height");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (width != null && height != null) {
                            SlangRetailAssistant.getUI().setTriggerSize(width, height);
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setTextColor":
                String textColor = SlangRetailAssistantUtils.getStringConfig("color");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (textColor != null) {
                            SlangRetailAssistant.getUI().setTextColor(textColor);
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setSecondaryTextColor":
                String secondaryTextColor = SlangRetailAssistantUtils.getStringConfig("color");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (secondaryTextColor != null) {
                            SlangRetailAssistant.getUI().setSecondaryTextColor(secondaryTextColor);
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setSettingsButtonBackgroundColor":
                String settingsButtonBackgroundColor = SlangRetailAssistantUtils.getStringConfig("color");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (settingsButtonBackgroundColor != null) {
                            SlangRetailAssistant.getUI().setSettingsButtonBackgroundColor(settingsButtonBackgroundColor);
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setControlButtonBackgroundColor":
                String controlButtonBackgroundColor = SlangRetailAssistantUtils.getStringConfig("color");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (controlButtonBackgroundColor != null) {
                            SlangRetailAssistant.getUI().setControlButtonBackgroundColor(controlButtonBackgroundColor);
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setTextHighlightColor":
                String textHighlightColor = SlangRetailAssistantUtils.getStringConfig("color");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        if (textHighlightColor != null) {
                            SlangRetailAssistant.getUI().setTextHighlightColor(textHighlightColor);
                        }
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setTriggerImageResource":
                Object obj = call.arguments;
                byte[] imageByteArray = (byte[]) obj;
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        Drawable image = new BitmapDrawable(application.getResources(),
                                BitmapFactory.decodeByteArray(imageByteArray, 0,
                                        imageByteArray.length));
                        SlangRetailAssistant.getUI().setTriggerImageDrawable(image);
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            // UI Methods end
            case "notifySearchActionCompleted":
                try {
                    String appStateString = call.argument("appState");
                    String appStateConditionString = call.argument("appStateCondition");

                    if (searchUserJourney != null) {
                        try {
                            SearchAppState searchAppState = getSearchAppState(appStateString,appStateConditionString);
                            searchUserJourney.notifyAppState(searchAppState);
                        } catch (Exception exception) {
                            sendAssistantError(exception);
                        }
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setUserJourneyToContinue":
                try {
                    String currentUserJourney = call.argument("currentUserJourney");
                    String userJourneyToContinue = call.argument("userJourneyToContinue");
                    Boolean isSpoken = call.argument("isSpoken");
                    if (isSpoken == null) {
                        isSpoken = false;
                    }
                    Log.d(TAG, "setUserJourneyToContinue");
                    RetailUserJourney assistantUserJourney = RetailUserJourney.fromString(currentUserJourney);
                    if (assistantUserJourney != null) {
                        switch (assistantUserJourney) {
                            case SEARCH:
                                if (searchUserJourney != null) {
                                    searchUserJourney.setRetailJourneyToContinue(RetailUserJourney
                                            .fromString(userJourneyToContinue), isSpoken);
                                }
                                break;
                            case NAVIGATION:
                                if (navigationUserJourney != null) {
                                    navigationUserJourney.setRetailJourneyToContinue(RetailUserJourney.fromString(userJourneyToContinue), isSpoken);
                                }
                                break;
                            case ORDER_MANAGEMENT:
                                if (orderManagementUserJourney != null) {
                                    orderManagementUserJourney.setRetailJourneyToContinue(RetailUserJourney.fromString(userJourneyToContinue), isSpoken);
                                }
                                break;
                            default:
                                break;
                        }
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "clearUserJourneyToContinue":
                try {
                    String currentUserJourney = call.argument("currentUserJourney");
                    Log.d(TAG, "clearUserJourneyToContinue");
                    RetailUserJourney assistantUserJourney = RetailUserJourney.fromString(currentUserJourney);
                    if (assistantUserJourney != null) {
                        switch (assistantUserJourney) {
                            case SEARCH:
                                if (searchUserJourney != null) {
                                    searchUserJourney.clearRetailJourneyToContinue();
                                }
                                break;
                            case NAVIGATION:
                                if (navigationUserJourney != null) {
                                    navigationUserJourney.clearRetailJourneyToContinue();
                                }
                                break;
                            case ORDER_MANAGEMENT:
                                if (orderManagementUserJourney != null) {
                                    orderManagementUserJourney.clearRetailJourneyToContinue();
                                }
                                break;
                            default:
                                break;
                        }
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setAppDefaultUserJourney":
                try {
                    String defaultUserJourney = call.argument("userJourney");
                    if (defaultUserJourney != null) {
                        SlangRetailAssistant.setAppDefaultUserJourney(RetailUserJourney.fromString(defaultUserJourney));
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setAppDefaultSubDomain":
                try {
                    String appSubDomain = call.argument("appSubDomain");
                    if (appSubDomain != null) {
                        SlangRetailAssistant.setAppDefaultSubDomain(AssistantSubDomain.fromString(appSubDomain));
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "clearSearchUserJourneyContext":
                try {
                    SearchUserJourney.getContext().clear();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "disableSearchUserJourneyContext":
                try {
                    SearchUserJourney.disablePreserveContext();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

//                NAVIGATION

            case "notifyNavigationActionCompleted":
                try {
                    String appStateString = call.argument("appState");
                    String appStateConditionString = call.argument("appStateCondition");

                    if (navigationUserJourney != null) {
                        try {
                            NavigationAppState navigationAppState = getNavigationAppState(appStateString,appStateConditionString);
                            navigationUserJourney.notifyAppState(navigationAppState);
                        } catch (Exception exception) {
                            sendAssistantError(exception);
                        }
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;


            case "disableNavigationUserJourneyContext":
                try {
                    NavigationUserJourney.disablePreserveContext();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "clearNavigationUserJourneyContext":
                try {
                    NavigationUserJourney.getContext().clear();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "notifyNonVoiceUserJourney":
                try {
                    RetailUserJourney nonVoiceUserJourney = RetailUserJourney.SEARCH;
                    String nonVoiceUserJourneyString = call.argument("userJourney");
                    if (nonVoiceUserJourneyString != null) {
                        nonVoiceUserJourney = RetailUserJourney.fromString(nonVoiceUserJourneyString);
                    }
                    Map<String, String> journeyInfo = new HashMap<>();
                    if (call.argument("userJourneyInfo") != null &&
                            call.argument("userJourneyInfo") instanceof Map) {
                        journeyInfo = call.argument("userJourneyInfo");
                    }
                    SlangRetailAssistant.notifyNonVoiceUserJourney(
                            nonVoiceUserJourney, journeyInfo);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "notifyNonVoiceSearch":
                try {
                    String searchString = call.argument("searchString");
                    SlangRetailAssistant.notifyNonVoiceSearch(searchString);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "trackCustomEvent":
                try {
                    String eventName = call.argument("eventName");
                    Map<String, String> eventMetaData = new HashMap<>();
                    if (call.argument("eventMetaData") != null && call.argument("eventMetaData") instanceof Map) {
                        eventMetaData = call.argument("eventMetaData");
                    }
                    SlangRetailAssistant.trackCustomEvent(eventName, eventMetaData);
                } catch (Exception e) {
                    sendAssistantError(e);
                }
                break;

//                ORDER MANAGEMENT

            case "cancelSession":
                try {
                    SlangBuddy.getBuiltinUI().cancel();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "notifyOrderManagementActionCompleted":
                try {
                    String appStateString = call.argument("appState");
                    String appStateConditionString = call.argument("appStateCondition");

                    if (orderManagementUserJourney != null) {
                        try {
                            OrderManagementAppState orderManagementAppState = getOrderManagementAppState(appStateString,appStateConditionString);
                            orderManagementUserJourney.notifyAppState(orderManagementAppState);
                        } catch (Exception exception) {
                            sendAssistantError(exception);
                        }
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;


            case "disableOrderManagementUserJourneyContext":
                try {
                    OrderManagementUserJourney.disablePreserveContext();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "clearOrderManagementUserJourneyContext":
                try {
                    OrderManagementUserJourney.getContext().clear();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setItemBrand":
                try {
                    String brand = SlangRetailAssistantUtils.getStringConfig("brand");
                    SearchUserJourney.getContext().setItemBrand(brand);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemCategory":
                try {
                    String category = SlangRetailAssistantUtils.getStringConfig(" category");
                    SearchUserJourney.getContext().setItemCategory(category);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemProductType":
                try {
                    String productType = SlangRetailAssistantUtils.getStringConfig("productType");
                    SearchUserJourney.getContext().setItemProductType(productType);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemDescription":
                try {
                    String description = SlangRetailAssistantUtils.getStringConfig("description");
                    SearchUserJourney.getContext().setItemDescription(description);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemPrice":
                try {
//                    Item.Price price = SlangRetailAssistantUtils.getPriceConfig("price");
//                    SearchUserJourney.getContext().setItemPrice(price);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemSize":
                try {
                    Item.Size size = SlangRetailAssistantUtils.getSizeConfig("size");
                    SearchUserJourney.getContext().setItemSize(size);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemVariants":
                try {
                    List<String> variants = SlangRetailAssistantUtils.getStringListConfig("variants");
                    SearchUserJourney.getContext().setItemVariants(variants.toArray(new String[variants.size()]));
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemQuantity":
                try {
                    Item.Quantity quantity = SlangRetailAssistantUtils.getQuantityConfig("quantity");
                    SearchUserJourney.getContext().setItemQuantity(quantity);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
            case "setItemId":
                try {
                    String id = SlangRetailAssistantUtils.getStringConfig("id");
                    SearchUserJourney.getContext().setItemId(id);
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

        }
    }
    @Override
    public void onDetachedFromEngine(@NonNull FlutterPlugin.FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        currentActivity = new WeakReference<>(binding.getActivity());
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }

    private void sendAssistantError(final Exception exception) {
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                Map<String, String> params = new HashMap<>();
                params.put("type", exception.getMessage());
                channel.invokeMethod("onAssistantError", params);
            }
        });
    }


}
