//
//  Copyright (c) 2017-2021 Slang Labs Private Limited. All rights reserved.
//  

import Foundation
import SlangRetailAssistant
import SlangAssistant

class SlangRetailAssistantUtils {
    
    private static var configOptions = Dictionary<String, Any>()
    
    static func setConfigOptions(_ configOptions:Dictionary<String,Any>) {
        self.configOptions = configOptions
    }
    
    static func getStringConfig(_ key:String) -> String {
        return configOptions[key] as? String ?? ""
    }
    
    static func getBooleanConfig(_ key:String) -> Bool {
        return configOptions[key] as? Bool ?? false
    }
    
    static func getStringListConfig(_ key:String) -> [String] {
        if let stringArray = configOptions[key] as? [String] {
            return stringArray
        }
        return [String]()
    }
    
    static func getAssistantUIConfig(_ key:String) -> BaseUIPosition {
        var uiPosition = AssistantUIPosition.AssistantUIPositionEnum.centerBottom
        var assistantUIPosition = AssistantUIPosition.Builder()
            .setBaseUIPosition(uiPosition)
            .setIsDraggable(false)
            .setIsForcedAtStartup(false)
            .build()
        
        guard let assistantUIMap = configOptions[key] as? [String:Any] else {
            return assistantUIPosition
        }
        
        let baseUIPosition = assistantUIMap["baseUIPosition"] as? String ?? ""
        switch baseUIPosition {
        case "BOTTOM_LEFT":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.leftBottom
        case "CENTER_LEFT":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.leftCenter
        case "CENTER":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.center
        case "CENTER_RIGHT":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.rightCenter
        case "TOP_LEFT":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.leftTop
        case "TOP_CENTER":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.centerTop
        case "TOP_RIGHT":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.rightTop
        case "BOTTOM_RIGHT":
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.rightBottom
        default:
            uiPosition = AssistantUIPosition.AssistantUIPositionEnum.centerBottom
        }
        
        let offsetX = assistantUIMap["offsetX"] as? Int
        let offsetY = assistantUIMap["offsetY"] as? Int
        let draggable = assistantUIMap["draggable"] as? Bool ?? false
        let forcedAtStartup = assistantUIMap["forcedAtStartup"] as? Bool ?? false
        
        assistantUIPosition = AssistantUIPosition.Builder()
            .setBaseUIPosition(uiPosition)
            .setIsDraggable(draggable)
            .setOffsetX(offsetX ?? 8)
            .setOffsetY(offsetY ?? 8)
            .setIsForcedAtStartup(forcedAtStartup)
            .build()
        
        return assistantUIPosition
    }
    
    static func mapSearchInfo(_ searchInfo:SearchInfo) -> [String:Any?] {
        var searchInfoMap = [String:Any?]()
        searchInfoMap["item"] = mapItem(searchInfo.item)
        searchInfoMap["filterInfoList"] = mapFilterInfoList(searchInfo.filterInfoList)
        searchInfoMap["sortingInfo"] = mapSortingInfo(searchInfo.sortingInfo)
        searchInfoMap["isAddToCart"] = searchInfo.isAddToCart
        return searchInfoMap
    }
    
    static func mapNavigationInfo(_ navigationInfo:NavigationInfo) -> [String:Any?] {
        var navigationInfoMap = [String:Any?]()
        navigationInfoMap["target"] = navigationInfo.target
        return navigationInfoMap
    }
    
    static func mapItem(_ item:Item?) -> [String:Any]? {
        var itemMap = [String:Any]()
        guard let item = item else {return nil}
        itemMap["id"] = item.itemId
        itemMap["brand"] = item.brand
        itemMap["productType"] = item.productType
        itemMap["description"] = item.getDescription()
        itemMap["category"] = item.category
        itemMap["variants"] = mapItemVariantList(item.variants)
        itemMap["quantity"] = mapItemQuantity(item.quantity)
        itemMap["size"] = mapItemSize(item.size)
        itemMap["price"] = mapItemPrice(item.price)
        return itemMap
    }
    
    private static func mapFilterInfoList(_ filterInfos:[FilterInfo]?) -> [Any]? {
        var filterInfoListArray = [Any]()
        guard let filterInfos = filterInfos else {return [Any]()}
        for filterInfo in filterInfos {
            filterInfoListArray.append(mapFilterInfo(filterInfo))
        }
        return filterInfoListArray
    }
    
    
    private static func  mapFilterInfo(_ filterInfo:FilterInfo?) -> [String:Any] {
        var filterInfoMap = [String:Any]()
        guard let filterInfo = filterInfo else {return filterInfoMap}
        filterInfoMap["filterName"] = filterInfo.filterName
        filterInfoMap["filterValue"] = filterInfo.filterValue
        return filterInfoMap
    }
    
    private static func mapSortingInfo(_ sortingInfo:SortingInfo?) -> [String:Any]? {
        var sortingInfoMap = [String:Any]()
        guard let sortingInfo = sortingInfo else {return nil}
        sortingInfoMap["sortOrder"] = sortingInfo.sortOrder.rawValue
        sortingInfoMap["sortType"] = sortingInfo.sortType.rawValue
        return sortingInfoMap
    }
    
    private static func mapItemVariantList(_ varaints:[String]?) -> [String]? {
        var variantsListArray = [String]()
        guard let variants = varaints else {return nil}
        for variant in variants {
            variantsListArray.append(variant)
        }
        return variantsListArray
    }
    
    private static func mapItemQuantity(_ itemQuantityInfo:ItemQuantity?) -> [String:Any]? {
        var itemQuantityMap = [String:Any]()
        guard let itemQuantityInfo = itemQuantityInfo else {return nil}
        itemQuantityMap["amount"] = itemQuantityInfo.amount
        itemQuantityMap["unit"] = itemQuantityInfo.unit.description
        return itemQuantityMap
    }
    
    private static func mapItemSize(_ itemSizeInfo:ItemSize?) -> [String:Any]? {
        var itemSizeMap = [String:Any]()
        guard let itemSizeInfo = itemSizeInfo else {return nil}
        itemSizeMap["amount"] = itemSizeInfo.amount
        itemSizeMap["unit"] = itemSizeInfo.unit.description
        return itemSizeMap
    }
    
    private static func mapItemPrice(_ itemPriceInfo:ItemPrice?) -> [String:Any]? {
        var itemPriceMap = [String:Any]()
        guard let itemPriceInfo = itemPriceInfo else {return nil}
        itemPriceMap["minAmount"] = itemPriceInfo.minAmount
        itemPriceMap["maxAmount"] = itemPriceInfo.maxAmount
        itemPriceMap["currency"] = itemPriceInfo.currency.description
        return itemPriceMap
    }
    
}
