library slang_retail_assistant;

import 'dart:collection';
import 'dart:core';
import 'dart:io' show Platform;

import 'package:flutter/services.dart';

part 'src/assistant_config.dart';

part 'src/assistant_ui.dart';

part 'src/assistant_ui_position.dart';

part 'src/navigation_info.dart';

part 'src/navigation_user_journey.dart';

part 'src/order_info.dart';

part 'src/order_management_user_journey.dart';

part 'src/search_info.dart';

part 'src/search_user_journey.dart';

part 'src/appState/SearchResultAppState.dart';

part 'src/appState/AddToCartAppState.dart';

part 'src/appState/UnSupportedAppState.dart';

part 'src/appState/WaitingAppState.dart';

part 'src/appState/OrderViewAppState.dart';

part 'src/appState/OrderCancelAppState.dart';

part 'src/appState/NavigationCompleteAppState.dart';

part 'src/appState/SearchAppState.dart';

part 'src/appState/BaseAppState.dart';

part 'src/appState/BaseCondition.dart';

part 'src/appState/RetailAppState.dart';

part 'src/appState/RetailCondition.dart';

part 'src/appState/Condition.dart';

part 'src/appState/OrderManagementAppState.dart';

part 'src/appState/NavigationAppState.dart';

class SlangRetailAssistant {
  SlangRetailAssistant._();

  static const MethodChannel _channel =
      const MethodChannel('plugins.flutter.io/slang_retail_assistant');

  static late RetailAssistantAction _action;
  static late ContextualOnboardingListener _contextualOnboardingAction;
  static late RetailAssistantLifeCycleObserver _lifeCycleObserver;
  static late SearchUserJourney _searchUserJourney;
  static late NavigationUserJourney _navigationUserJourney;
  static late OrderManagementUserJourney _orderManagementUserJourney;

  static Future<void> _methodCallHandler(MethodCall call) async {
    switch (call.method) {
      case 'onSearch':
        _searchUserJourney = new SearchUserJourney._();
        SearchAppState searchAppState = _action.onSearch(
            new SearchInfo._(call.arguments), _searchUserJourney);
        if (searchAppState != new WaitingAppState()) {
          _searchUserJourney.notifyAppState(searchAppState);
        }
        break;
      case 'onNavigation':
        _navigationUserJourney = new NavigationUserJourney._();
        NavigationAppState navigationAppState = _action.onNavigation(
            NavigationInfo._(call.arguments), _navigationUserJourney);
        if (navigationAppState != new WaitingAppState()) {
          _navigationUserJourney.notifyAppState(navigationAppState);
        }
        break;
      case 'onOrderManagement':
        _orderManagementUserJourney = new OrderManagementUserJourney._();
        OrderManagementAppState orderManagementAppState =
            _action.onOrderManagement(
                OrderInfo._(call.arguments), _orderManagementUserJourney);
        if (orderManagementAppState != new WaitingAppState()) {
          _orderManagementUserJourney.notifyAppState(orderManagementAppState);
        }
        break;
      case 'onAssistantError':
        final Map<dynamic, dynamic> _data = call.arguments;
        Map<String, String> arguments = new HashMap();
        arguments["description"] = _data["description"];
        _action.onAssistantError(arguments);
        break;
      case 'onAssistantInitSuccess':
        _lifeCycleObserver.onAssistantInitSuccess();
        break;
      case 'onAssistantInitFailure':
        final Map<dynamic, dynamic> _data = call.arguments;
        String description = _data["type"];
        _lifeCycleObserver.onAssistantInitFailure(description);
        break;
      case 'onAssistantInvoked':
        _lifeCycleObserver.onAssistantInvoked();
        break;
      case 'onAssistantClosed':
        final Map<dynamic, dynamic> _data = call.arguments;
        bool isCancelled = _data["isCancelled"];
        _lifeCycleObserver.onAssistantClosed(isCancelled);
        break;
      case 'onAssistantLocaleChanged':
        final Map<dynamic, dynamic> _data = call.arguments;
        Map<String, String> locale = new HashMap();
        locale["country"] = _data["country"];
        locale["language"] = _data["language"];
        _lifeCycleObserver.onAssistantLocaleChanged(locale);
        break;
      case 'onUnrecognisedUtterance':
        final Map<dynamic, dynamic> _data = call.arguments;
        String utterance = _data["utterance"];
        _lifeCycleObserver.onUnrecognisedUtterance(utterance);
        break;
      case 'onUtteranceDetected':
        final Map<dynamic, dynamic> _data = call.arguments;
        String utterance = _data["utterance"];
        _lifeCycleObserver.onUtteranceDetected(utterance);
        break;
      case 'onOnboardingSuccess':
        _lifeCycleObserver.onOnboardingSuccess();
        break;
      case 'onOnboardingFailure':
        _lifeCycleObserver.onOnboardingFailure();
        break;
      case 'onOnboardingFailure':
        _lifeCycleObserver.onOnboardingFailure();
        break;
      case 'onMicPermissionDenied':
        _lifeCycleObserver.onMicPermissionDenied();
        break;
      case 'onMicPermissionGranted':
        _lifeCycleObserver.onMicPermissionGranted();
        break;
      case 'onCoachmarkAction':
        final Map<dynamic, dynamic> _data = call.arguments;
        String coachmarkTypeString = _data["coachmarkType"];
        String coachmarkActionString = _data["coachmarkAction"];
        AssistantCoachmarkType coachmarkType = AssistantCoachmarkType.values
            .firstWhere((e) =>
                e.toString() ==
                'AssistantCoachmarkType.' + coachmarkTypeString);
        AssistantCoachmarkAction coachmarkAction =
            AssistantCoachmarkAction.values.firstWhere((e) =>
                e.toString() ==
                'AssistantCoachmarkAction.' + coachmarkActionString);
        _lifeCycleObserver.onCoachmarkAction(coachmarkType, coachmarkAction);
        break;
      case 'onContextualOnboardingResponse':
        final Map<dynamic, dynamic> _data = call.arguments;
        String contextualOnboardingResponseString =
            _data["contextualOnboardingResponse"];
        String contextualOnboardingResponseReasonString =
            _data["contextualOnboardingResponseReason"];
        ContextualOnboardingResponse contextualOnboardingResponse =
            ContextualOnboardingResponse.values.firstWhere((e) =>
                e.toString() ==
                'ContextualOnboardingResponse.' +
                    contextualOnboardingResponseString);
        ContextualOnboardingResponseReason contextualOnboardingResponseReason =
            ContextualOnboardingResponseReason.values.firstWhere((e) =>
                e.toString() ==
                'ContextualOnboardingResponseReason.' +
                    contextualOnboardingResponseReasonString);
        _contextualOnboardingAction(
            contextualOnboardingResponse, contextualOnboardingResponseReason);
        break;
      default:
        print(
            'SlangRetailAssistant: Ignoring invoke from native. This normally shouldn\'t happen.');
    }
  }

  static bool _validateEnvironment() {
    return !Platform.isIOS;
  }

  static void initialize(AssistantConfiguration assistantConfiguration) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod(
        "initialize", assistantConfiguration._covertToHashmap());
    _channel.setMethodCallHandler(_methodCallHandler);
  }

  static void setAction(RetailAssistantAction action) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod("setAction");
    _action = action;
  }

  static void setLifecycleObserver(
      RetailAssistantLifeCycleObserver lifeCycleObserver) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod("setLifecycleObserver");
    _lifeCycleObserver = lifeCycleObserver;
  }

  static void startConversation(AssistantUserJourney assistantUserJourney) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod(
        "startConversation", {"userJourney": assistantUserJourney.stringValue});
  }

  static void startContextualOnboarding(
      ContextualOnboardingListener contextualOnboardingListener) {
    if (!_validateEnvironment()) {
      return;
    }
    _contextualOnboardingAction = contextualOnboardingListener;
    _channel.invokeMapMethod("startContextualOnboarding");
  }

  static void notifyNonVoiceUserJourney(
      AssistantUserJourney assistantUserJourney,
      Map<String, String> journeyInfo) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod("notifyNonVoiceUserJourney", {
      "userJourney": assistantUserJourney.stringValue,
      "userJourneyInfo": journeyInfo
    });
  }

  static void notifyNonVoiceSearch(String searchString) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod(
        "notifyNonVoiceSearch", {"searchString": searchString});
  }

  static void trackCustomEvent(
      String eventName, Map<String, String> eventMetaData) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod("trackCustomEvent",
        {"eventName": eventName, "eventMetaData": eventMetaData});
  }

  static void cancelSession() {
    _channel.invokeMapMethod("cancelSession");
  }

  static _AssistantUI getUI() {
    return _AssistantUI(_channel);
  }
}

class RetailAssistantAction {
  SearchAppState onSearch(
      SearchInfo searchInfo, SearchUserJourney searchUserJourney) {
    return new UnSupportedAppState();
  }

  NavigationAppState onNavigation(NavigationInfo navigationInfo,
      NavigationUserJourney navigationUserJourney) {
    return new UnSupportedAppState();
  }

  OrderManagementAppState onOrderManagement(OrderInfo orderInfo,
      OrderManagementUserJourney orderManagementUserJourney) {
    return new UnSupportedAppState();
  }

  void onAssistantError(Map<String, String> assistantError) {}
}

typedef ContextualOnboardingListener = void Function(
  ContextualOnboardingResponse userResponse,
  ContextualOnboardingResponseReason responseReason,
);

enum AssistantCoachmarkType {
  UNKNOWN,
  ONBOARDING,
  USER_HINT,
  LOCALE_CHANGE_HINT
}

enum AssistantCoachmarkAction { UNKNOWN, VIEW, CANCEL, CLICK }

enum ContextualOnboardingResponse {
  CONTINUE_WITH_VOICE,
  CONTINUE_WITHOUT_VOICE
}

enum ContextualOnboardingResponseReason {
  USER_PROCESSING_ONBOARDING,
  USER_SELECTED_VOICE,
  USER_COMPLETED_ONBOARDING,
  USER_DISMISSED_ONBOARDING,
  USER_DENIED_MICROPHONE_PERMISSION,
  INVALID_STATE
}

class RetailAssistantLifeCycleObserver {
  void onAssistantInitSuccess() {}

  void onAssistantInitFailure(String description) {}

  void onAssistantInvoked() {}

  void onAssistantClosed(bool isCancelled) {}

  void onAssistantLocaleChanged(Map<String, String> locale) {}

  void onUnrecognisedUtterance(String utterance) {}

  void onUtteranceDetected(String utterance) {}

  void onOnboardingSuccess() {}

  void onOnboardingFailure() {}

  void onMicPermissionDenied() {}

  void onMicPermissionGranted() {}

  void onCoachmarkAction(AssistantCoachmarkType coachmarkType,
      AssistantCoachmarkAction coachmarkAction) {}
}

enum AssistantUserJourney { SEARCH, NAVIGATION, ORDER_MANAGEMENT }

extension AssistantUserJourneyExtention on AssistantUserJourney {
  String get stringValue {
    switch (this) {
      case AssistantUserJourney.SEARCH:
        return 'retail_search';
      case AssistantUserJourney.NAVIGATION:
        return 'retail_navigation';
      case AssistantUserJourney.ORDER_MANAGEMENT:
        return 'retail_order_management';
      default:
        return "retail_search";
    }
  }
}

enum AssistantSubDomain { GROCERY, PHARMACY, CUSTOM }

extension AssistantSubDomainExtension on AssistantSubDomain {
  String get stringValue {
    switch (this) {
      case AssistantSubDomain.GROCERY:
        return 'grocery';
      case AssistantSubDomain.PHARMACY:
        return 'pharmacy';
      case AssistantSubDomain.CUSTOM:
        return 'custom';
      default:
        return "grocery";
    }
  }
}

enum AssistantEnvironment { STAGING, PRODUCTION }

extension AssistantEnvironmentExtension on AssistantEnvironment {
  String get stringValue {
    switch (this) {
      case AssistantEnvironment.STAGING:
        return 'STAGING';
      case AssistantEnvironment.PRODUCTION:
        return 'PRODUCTION';
      default:
        return "STAGING";
    }
  }
}

enum AssistantTriggerStyle { FLAT, DEFAULT }

extension AssistantTriggerStyleExtension on AssistantTriggerStyle {
  String get stringValue {
    switch (this) {
      case AssistantTriggerStyle.DEFAULT:
        return 'DEFAULT';
      default:
        return 'FLAT';
    }
  }
}

enum AssistantSurfaceStyle { CARD, CURVED, GRADIENT, MINIMAL_CARD }

extension AssistantSurfaceStyleExtension on AssistantSurfaceStyle {
  String get stringValue {
    switch (this) {
      case AssistantSurfaceStyle.GRADIENT:
        return 'GRADIENT';
      case AssistantSurfaceStyle.CURVED:
        return 'CURVED';
      case AssistantSurfaceStyle.MINIMAL_CARD:
        return 'MINIMAL_CARD';
      default:
        return 'CARD';
    }
  }
}
