part of slang_retail_assistant;

class SearchInfo {
  Item? _item;
  late bool _isAddToCart;
  SortingInfo? _sortingInfo;
  List<FilterInfo>? _filterInfoList;

  Item? get item {
    return _item;
  }

  bool get isAddToCart {
    return _isAddToCart;
  }

  SortingInfo? get sortingInfo {
    return _sortingInfo;
  }

  List<FilterInfo>? get filterInfoList {
    return _filterInfoList;
  }

  SearchInfo._(Map<dynamic, dynamic> map) {
    if (map["item"] != null) {
      _item = Item._(map["item"]);
    }
    _isAddToCart = map["isAddToCart"] ?? false;
    if (map["sortingInfo"] != null) {
      _sortingInfo = SortingInfo._(map["sortingInfo"]);
    } else {
      _sortingInfo = null;
    }
    if (map["filterInfoList"] != null) {
      List<dynamic>? filterInfos = map["filterInfoList"];
      List<FilterInfo> filterInfoListArray = [];
      if (filterInfos != null) {
        for (Map<dynamic, dynamic> filterInfo in filterInfos) {
          filterInfoListArray.add(FilterInfo._(filterInfo));
        }
      }
      _filterInfoList = filterInfoListArray;
    }
  }

  Map toJson() => {
        'item': _item,
        'isAddToCart': _isAddToCart,
        'filterInfoList': _filterInfoList,
        'sortInfo': _sortingInfo
      };

  @override
  String toString() {
    return 'SearchInfo{_item: ${_item.toString()}, _isAddToCart: $_isAddToCart, _sortingInfo: $_sortingInfo, _filterInfoList: $_filterInfoList}';
  }
}

class Item {
  String? _description;
  String? _id;
  String? _brand;
  String? _productType;
  String? _category;
  List<String>? _variants;
  List<String>? _productNames;
  Quantity? _quantity;
  Price? _price;
  Size? _size;

  String? get description {
    return _description;
  }

  String? get id {
    return _id;
  }

  String? get brand {
    return _brand;
  }

  String? get productType {
    return _productType;
  }

  String? get category {
    return _category;
  }

  List<String>? get variants {
    return _variants;
  }

  List<String>? get productNames {
    return _productNames;
  }

  Quantity? get quantity {
    return _quantity;
  }

  Price? get price {
    return _price;
  }

  Size? get size {
    return _size;
  }

  Item._(Map<dynamic, dynamic> map) {
    _description = map["description"];
    _id = map["id"];
    _brand = map["brand"];
    _productType = map["productType"];
    _category = map["category"];
    List<String> variantsString = [];
    List<dynamic>? variantList = map["variants"];
    if (variantList != null) {
      for (String variant in variantList) {
        variantsString.add(variant);
      }
    }
    _variants = variantsString;
    List<String> productNamesString = [];
    List<dynamic>? productNames = map["productNames"];
    if (productNames != null) {
      for (String productName in productNames) {
        productNamesString.add(productName);
      }
    }
    _productNames = productNamesString;
    if (map["quantity"] != null) {
      _quantity = Quantity._(map["quantity"]);
    }
    if (map["size"] != null) {
      _size = Size._(map["size"]);
    }
    if (map["price"] != null) {
      _price = Price._(map["price"]);
    }
  }

  Map toJson() => {
        'description': _description,
        'id': _id,
        'brand': _brand,
        'productType': _productType,
        'category': _category,
        'variants': _variants,
        'productNames': _productNames,
        'quantity': _quantity,
        'size': _size,
        'price': _price
      };

  @override
  String toString() {
    return 'Item{_description: $_description, _id: $_id, _brand: $_brand, _productType: $_productType, _category: $_category, _variants: $_variants, _quantity: $_quantity, _price: $_price, _size: $_size}';
  }
}

class Quantity {
  late int _amount;
  late QuantityUnit _unit;

  int get amount {
    return _amount;
  }

  QuantityUnit get unit {
    return _unit;
  }

  Quantity._(Map<dynamic, dynamic> map) {
    _amount = map["amount"];

    var unit = map['unit'] ?? "";
    switch (unit) {
      case "":
        this._unit = QuantityUnit.UNKNOWN;
        break;
      case "items":
        this._unit = QuantityUnit.ITEM;
        break;
      case "packs":
        this._unit = QuantityUnit.PACK;
        break;
      case "numbers":
        this._unit = QuantityUnit.NUMBER;
        break;
      case "sachets":
        this._unit = QuantityUnit.SACHET;
        break;
      case "boxes":
        this._unit = QuantityUnit.BOX;
        break;
      case "pieces":
        this._unit = QuantityUnit.PIECE;
        break;
      case "bottles":
        this._unit = QuantityUnit.BOTTLE;
        break;
      default:
        this._unit = QuantityUnit.UNKNOWN;
    }
  }

  @override
  String toString() {
    return amount.toString() + " " + _unit.name;
  }

  Map toJson() => {'amount': _amount, 'unit': _unit.toString()};
}

enum QuantityUnit { UNKNOWN, ITEM, PACK, NUMBER, SACHET, BOX, PIECE, BOTTLE }

extension QuantityUnitExtension on QuantityUnit {
  String get name {
    switch (this) {
      case QuantityUnit.ITEM:
        return 'items';
      case QuantityUnit.PACK:
        return 'packs';
      case QuantityUnit.NUMBER:
        return 'numbers';
      case QuantityUnit.SACHET:
        return 'sachets';
      case QuantityUnit.BOX:
        return 'boxes';
      case QuantityUnit.PIECE:
        return 'pieces';
      case QuantityUnit.BOTTLE:
        return 'bottles';
      default:
        return "";
    }
  }
}

class Size {
  late int _amount;
  late SizeUnit _unit;

  int get amount {
    return _amount;
  }

  SizeUnit get unit {
    return _unit;
  }

  Size(this._amount, this._unit);

  Size._(Map<dynamic, dynamic> map) {
    _amount = map["amount"];

    var unit = map['unit'] ?? "";
    switch (unit) {
      case "":
        this._unit = SizeUnit.UNKNOWN;
        break;
      case "ml":
        this._unit = SizeUnit.MILLILITRE;
        break;
      case "l":
        this._unit = SizeUnit.LITRE;
        break;
      case "gm":
        this._unit = SizeUnit.GRAM;
        break;
      case "kg":
        this._unit = SizeUnit.KILOGRAM;
        break;
      default:
        this._unit = SizeUnit.UNKNOWN;
    }
  }

  @override
  String toString() {
    return amount.toString() + " " + _unit.name;
  }

  Map toJson() => {'amount': _amount, 'unit': _unit.toString()};
}

enum SizeUnit { UNKNOWN, MILLILITRE, LITRE, GRAM, KILOGRAM }

extension SizeUnitExtension on SizeUnit {
  String get name {
    switch (this) {
      case SizeUnit.KILOGRAM:
        return 'kg';
      case SizeUnit.GRAM:
        return 'g';
      case SizeUnit.LITRE:
        return 'l';
      case SizeUnit.MILLILITRE:
        return 'ml';
      default:
        return "";
    }
  }
}

class Price {
  late int _minAmount;
  late int _maxAmount;
  late PriceCurrency _currency;

  int get minAmount {
    return _minAmount;
  }

  int get maxAmount {
    return _maxAmount;
  }

  PriceCurrency get currency {
    return _currency;
  }

  Price._(Map<dynamic, dynamic> map) {
    _minAmount = map["minAmount"];
    _maxAmount = map["maxAmount"];

    var currency = map['currency'] ?? "";
    switch (currency) {
      case "":
        this._currency = PriceCurrency.UNKNOWN;
        break;
      case "rupees":
        this._currency = PriceCurrency.INR;
        break;
      case "dollars":
        this._currency = PriceCurrency.USD;
        break;
      default:
        this._currency = PriceCurrency.UNKNOWN;
    }
  }

  Map toJson() => {
        'minAmount': _minAmount,
        'maxAmount': _maxAmount,
        'currency': _currency.toString()
      };
}

enum PriceCurrency { UNKNOWN, INR, USD }

class FilterInfo {
  late String _filterName;
  late String _filterValue;

  String get filterName {
    return _filterName;
  }

  String get filterValue {
    return _filterValue;
  }

  FilterInfo._(Map<dynamic, dynamic> map) {
    _filterName = map["filterName"];
    _filterValue = map["filterValue"];
  }

  Map toJson() => {'filterName': _filterName, 'filterValue': _filterValue};
}

class SortingInfo {
  late SortType _sortType;
  late SortOrder _sortOrder;

  SortType get sortType {
    return _sortType;
  }

  SortOrder get sortOrder {
    return _sortOrder;
  }

  SortingInfo._(Map<dynamic, dynamic> map) {
    var sortType = map['sortType'] ?? "";
    switch (sortType) {
      case "unknown":
        this._sortType = SortType.UNKNOWN;
        break;
      case "relevance":
        this._sortType = SortType.RELEVANCE;
        break;
      case "popularity":
        this._sortType = SortType.POPULARITY;
        break;
      case "price":
        this._sortType = SortType.PRICE;
        break;
      case "recommendation":
        this._sortType = SortType.RECOMMENDATION;
        break;
      case "discount":
        this._sortType = SortType.DISCOUNT;
        break;
      case "newness":
        this._sortType = SortType.NEWNESS;
        break;
      default:
        this._sortType = SortType.UNKNOWN;
    }

    var sortOrder = map['sortOrder'] ?? "";
    switch (sortOrder) {
      case "unknown":
        this._sortOrder = SortOrder.UNKNOWN;
        break;
      case "ascending":
        this._sortOrder = SortOrder.ASCENDING;
        break;
      case "descending":
        this._sortOrder = SortOrder.DESCENDING;
        break;
      default:
        this._sortOrder = SortOrder.UNKNOWN;
    }
  }

  Map toJson() =>
      {'sortType': _sortType.toString(), 'sortOrder': _sortOrder.toString()};
}

enum SortType {
  UNKNOWN,
  RELEVANCE,
  POPULARITY,
  PRICE,
  RECOMMENDATION,
  DISCOUNT,
  NEWNESS
}

enum SortOrder { UNKNOWN, ASCENDING, DESCENDING }
